/*
 * Copyright (c) 1992, The Geometry Center
 *                     University of Minnesota 
 *                     1300 South Second Street
 *                     Minneapolis, MN  55454
 *
 * email address: software@geom.umn.edu
 *
 * This software is copyrighted as noted above.  It may be freely copied,
 * modified, and redistributed, provided that the copyright notice is
 * preserved on all copies.
 *
 * There is no warranty or other guarantee of fitness for this software,
 * it is provided solely "as is".  Bug reports or fixes may be sent
 * to the authors, who may or may not act on them as they desire.
 *
 * You may not include this software in a program or other software product
 * without supplying the source, or without informing the end-user that the
 * source is available for no extra charge.
 *
 * If you modify this software, you should include a notice giving the
 * name of the person performing the modification, the date of modification,
 * and the reason for such modification.
 *
 *     The National Science and Technology Research Center for
 *      Computation and Visualization of Geometric Structures
 */

#import <objc/objc.h>
#import <appkit/Application.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "link.h"

/*
#define DEBUG
*/
	char *machine_name = NULL;
	int which_dvc = 1;
  	char *infile = NULL;
  	char *inlist = NULL;
  	char *outfile = NULL;
	extern int xsegs, ysegs;
        short draw_all_rots=0, draw_all_regs=0;
        int ncan;
        region *canregs[MAX_CRS];  /* used by find_canonical_region to send all
				      canonical regions when draw_all_regs is set */
	char devicenames[][30] = { "postscript", "sunview","pixar", "mex"};
	char local[10] = "local";
	char inlistdef[] = "/tmp/linkreadxx";
        FILE *ifp, *lfp;
	struct rect {
		float xmin, xmax, ymin, ymax;
		} cnvs = {300,1000, 100, 800}; 
