/*
 * Copyright (c) 1992, The Geometry Center
 *                     University of Minnesota 
 *                     1300 South Second Street
 *                     Minneapolis, MN  55454
 *
 * email address: software@geom.umn.edu
 *
 * This software is copyrighted as noted above.  It may be freely copied,
 * modified, and redistributed, provided that the copyright notice is
 * preserved on all copies.
 *
 * There is no warranty or other guarantee of fitness for this software,
 * it is provided solely "as is".  Bug reports or fixes may be sent
 * to the authors, who may or may not act on them as they desire.
 *
 * You may not include this software in a program or other software product
 * without supplying the source, or without informing the end-user that the
 * source is available for no extra charge.
 *
 * If you modify this software, you should include a notice giving the
 * name of the person performing the modification, the date of modification,
 * and the reason for such modification.
 *
 *     The National Science and Technology Research Center for
 *      Computation and Visualization of Geometric Structures
 */

/* Generated by Interface Builder */

#import <appkit/View.h>

@interface FullView:View
{
    id	pageForm;
    id	distributor;
    id	dimensionsForm;
    id	pageText;
    id  viewList;

    int currentPage;
    int oldCP;       /* To remember currentPage while printing */

    char *filename;
}

- prevNext:sender;
- formChanged:sender;
- gotoPage:(int) page;
- newPage:sender;
- addPage;
- initFrame:(const NXRect *)frameRect;
- setDims:sender;
- getCurrentView;
- place:sender;
- placeList:newlist :lablist in:view;

/* File */
- save;
- saveAs;
- doSave;
- saveList: (NXTypedStream *) tstream;
- openList: (NXTypedStream *) tstream :(const char *) filenm;

- (BOOL)getRect:(NXRect *)theRect forPage:(int)page;
- (BOOL)knowsPagesFirst:(int *)firstPageNum last:(int *)lastPageNum;

@end
