/*
 * Copyright (c) 1992, The Geometry Center
 *                     University of Minnesota 
 *                     1300 South Second Street
 *                     Minneapolis, MN  55454
 *
 * email address: software@geom.umn.edu
 *
 * This software is copyrighted as noted above.  It may be freely copied,
 * modified, and redistributed, provided that the copyright notice is
 * preserved on all copies.
 *
 * There is no warranty or other guarantee of fitness for this software,
 * it is provided solely "as is".  Bug reports or fixes may be sent
 * to the authors, who may or may not act on them as they desire.
 *
 * You may not include this software in a program or other software product
 * without supplying the source, or without informing the end-user that the
 * source is available for no extra charge.
 *
 * If you modify this software, you should include a notice giving the
 * name of the person performing the modification, the date of modification,
 * and the reason for such modification.
 *
 *     The National Science and Technology Research Center for
 *      Computation and Visualization of Geometric Structures
 */

/* Constants which determine whether we're writing to a typed stream
   or a regular opened file */

#define DEST_FILE 0
#define DEST_TS   1

/* "dirty work" i/o functions */

int LinkSaveAll(LinkStatus *status,const char *name);
void saveMatrix(id linkObj, const char *filename);
void loadMatrix(id linkObj, const char *filename, int **rowptr,
		int ***matptr, int *numcrossptr);
int saveSB(LinkStatus *gnrc, const char *name);
int LinkSaveAll2(LinkStatus *status, void *fp, int dest);
int LinkPrintLink(LinkStatus *gnrc, void *fp, int dest, LinkList *lnk);
int LinkGetFile(LinkStatus *status,const char *name, NXZone *zone);
int LinkGetFile2(LinkStatus *status, void *fp, int dest, NXZone *zone);
int LinkReadStrand(LinkStatus *gnrc, void *fp, int dest, LinkList *lnk, NXZone *zone);
char *makeTitle(const char *filename);
